from django import forms
from django.core.files import File
from django.conf import settings
from io import StringIO

try:
    from .utils import FilepickerFile
    from .widgets import FPFileWidget
except ImportError:
    from utils import FilepickerFile
    from widgets import FPFileWidget

class FPFieldMixin():
    widget = FPFileWidget
    default_mimetypes = "*/*"

    def initialize(self, apikey=None, mimetypes=None, services=None, additional_params=None):
        """
        Initializes the Filepicker field.
        Valid arguments:
        * apikey. This string is required if it isn't set as settings.FILEPICKER_API_KEY
        * mimetypes. Optional, the allowed mimetypes for files. Defaults to "*/*" (all files)
        * services. Optional, the allowed services to pull from.
        * additional_params. Optional, additional parameters to be applied.
        """

        self.apikey = apikey or getattr(settings, 'FILEPICKER_API_KEY', None)
        if not self.apikey:
            raise Exception("Cannot find filepicker.io api key." +
            " Be sure to either pass as the apikey argument when creating the FPFileField," +
            " or set it as settings.FILEPICKER_API_KEY. To get a key, go to https://filepicker.io")

        self.mimetypes = mimetypes or self.default_mimetypes
        if not isinstance(self.mimetypes, str):
            #If mimetypes is an array, form a csv string
            try:
                self.mimetypes = ",".join(iter(self.mimetypes))
            except TypeError:
                self.mimetypes = str(self.mimetypes)

        self.services = services or getattr(settings, 'FILEPICKER_SERVICES', None)
        self.additional_params = additional_params or getattr(settings, 'FILEPICKER_ADDITIONAL_PARAMS', None)

    def widget_attrs(self, widget):
        attrs = {
                'data-fp-apikey': self.apikey,
                'data-fp-mimetypes': self.mimetypes,
                }

        if self.services:
            attrs['data-fp-option-services'] = self.services

        if self.additional_params:
            attrs = dict(list(attrs.items()) + list(self.additional_params.items()))            

        return attrs


class FPUrlField(FPFieldMixin, forms.URLField):
    widget = FPFileWidget
    default_mimetypes = "*/*"

    def __init__(self, *args, **kwargs):
        """
        Initializes the Filepicker url field.
        Valid arguments:
        * apikey. This string is required if it isn't set as settings.FILEPICKER_API_KEY
        * mimetypes. Optional, the allowed mimetypes for files. Defaults to "*/*" (all files)
        * services. Optional, the allowed services to pull from.
        * additional_params. Optional, additional parameters to be applied.
        """
        self.initialize(
            apikey=kwargs.pop('apikey', None),
            mimetypes=kwargs.pop('mimetypes', None),
            services=kwargs.pop('services', None),
            additional_params=kwargs.pop('additional_params', None),
        )
        super(FPUrlField, self).__init__(*args, **kwargs)


class FPFileField(FPFieldMixin, forms.FileField):
    def __init__(self, *args, **kwargs):
        """
        Initializes the Filepicker url field.
        Valid arguments:
        * apikey. This string is required if it isn't set as settings.FILEPICKER_API_KEY
        * mimetypes. Optional, the allowed mimetypes for files. Defaults to "*/*" (all files)
        * services. Optional, the allowed services to pull from.
        * additional_params. Optional, additional parameters to be applied.
        """
        self.initialize(
            apikey=kwargs.pop('apikey', None),
            mimetypes=kwargs.pop('mimetypes', None),
            services=kwargs.pop('services', None),
            additional_params=kwargs.pop('additional_params', None),
        )
        super(FPFileField, self).__init__(*args, **kwargs)

    def to_python(self, data):
        """Takes the url in data and creates a File object"""
        try:
            fpf = FilepickerFile(data)
        except ValueError as e:
            if 'Not a filepicker.io URL' in str(e):
                # Return None for invalid URLs
                return None
            else:
                # Pass the buck
                raise e
        else:
            return fpf.get_file(self.additional_params)



import re
import os
import requests
import tempfile
from django.core.files import File


class FilepickerFile(File):
    filepicker_url_regex = re.compile(
            r'https?:\/\/www.filepicker.io\/api\/file\/.*')

    def __init__(self, url):
        if not self.filepicker_url_regex.match(url):
            raise ValueError('Not a filepicker.io URL: %s' % url)
        self.url = url

    def get_file(self, additional_params={}):
        '''
        Downloads the file from filepicker.io and returns a
        Django File wrapper object.
        additional_params should include key/values such as:
        {
          'data-fp-signature': HEXDIGEST,
          'data-fp-policy': HEXDIGEST,
        }
        (in other words, parameters should look like additional_params
        of the models)
        '''
        # clean up any old downloads that are still hanging around
        self.cleanup()

        # Fetch any fields possibly required for fetching files for reading.
        query_params = {}

        if additional_params:
            for field in ('policy','signature'):
                longfield = 'data-fp-{0}'.format(field)
                if longfield in additional_params:
                    query_params[field] = additional_params[longfield]

        # Append the fields as GET query parameters to the URL in data.
        r = requests.get(self.url, params=query_params, stream=True)
        header = r.headers
        disposition = header.get('Content-Disposition')
        if disposition:
            name = disposition.rpartition("filename=")[2].strip('" ')
        filename = header.get('X-File-Name')
        if filename:
            name = filename

        # Create a temporary file to save to it later
        tmp = tempfile.NamedTemporaryFile(mode='w+b')
        for chunk in r.iter_content(chunk_size=1024):
            if chunk:
                tmp.write(chunk) # Write the chunk
                tmp.flush()

        # initialize File components of this object
        super(FilepickerFile, self).__init__(tmp,name=name)
        return self

    def cleanup(self):
        '''
        Removes any downloaded objects and closes open files.
        '''
        # self.file comes from Django File
        if hasattr(self, 'file'):
            if not self.file.closed:
                self.file.close()
            delattr(self, 'file')

        if hasattr(self, 'filename'):
            # the file might have been moved in the meantime so
            # check first
            if os.path.exists(self.filename):
                os.remove(self.filename)
            delattr(self, 'filename')

    def __enter__(self):
        '''
        Allow FilepickerFile to be used as a context manager as such:

            with FilepickerFile(url) as f:
                model.field.save(f.name, f.)
        '''
        self.get_file()
        # call Django's File context manager
        return super(FilepickerFile, self).__enter__()

    def __exit__(self, *args):
        # call Django's File context manager
        super(FilepickerFile, self).__exit__(*args)
        self.cleanup()

    def __del__(self):
        self.cleanup()



try:
    from .utils import FilepickerFile
except ImportError:
    from utils import FilepickerFile

class URLFileMapperMiddleware(object):
    """
    This middleware will take any Filepicker.io urls that are posted to the server via a POST
    and put a matching File object into request.FILES. This way, if you're used to grabbing files out of
    request.FILES, you don't have to change your backend code when using the filepicker.io widgets.

    This middleware is rather agressive in that it will automatically fetch any and all filepicker
    urls passed to the server, so if you are already processing the files via FPFileField or similar
    this functionality is redundant

    Note that the original filepicker.io url will still be available in POST if you need it.
    """
    def process_request(self, request):
        #Iterate over GET or POST data, search for filepicker.io urls
        for key, val in list(request.POST.items()):
            try:
                fp = FilepickerFile(val)
            except ValueError:
                pass
            else:
                splits = val.split(",")
                for url in splits:
                    if key in request.FILES:
                        request.FILES.setlist(key, list(
                            request.FILES.getlist(key) + [fp.get_file()]))
                    else:
                        request.FILES[key] = fp.get_file()



try:
	from . import models
	from . import forms
	from . import middleware
	from . import widgets
	from . import context_processors
except ImportError:
	import models
	import forms
	import middleware
	import widgets
	import context_processors